"""
Django admin declarations
"""

# Django
from django.contrib import admin

# AA SRP
from aasrp.models import AaSrpLink, AaSrpRequest, AaSrpRequestComment


def custom_filter(title):
    """
    custom filter for model properties
    :param title:
    :return:
    """

    class Wrapper(admin.FieldListFilter):
        """
        custom_filter :: wrapper
        """

        def __new__(cls, *args, **kwargs):
            instance = admin.FieldListFilter.create(*args, **kwargs)
            instance.title = title

            return instance

    return Wrapper


@admin.register(AaSrpLink)
class AaSrpLinkAdmin(admin.ModelAdmin):
    """
    AaSrpLinkAdmin
    """

    list_display = (
        "srp_code",
        "fleet_time",
        "_creator",
        "srp_name",
        "srp_status",
        "fleet_doctrine",
    )
    ordering = ("fleet_time",)

    list_filter = ("creator", "srp_status", "fleet_doctrine")

    search_fields = ("srp_code", "fleet_doctrine", "srp_name")

    @classmethod
    def _creator(cls, obj):
        creator_name = obj.creator
        if obj.creator.profile.main_character:
            creator_name = obj.creator.profile.main_character.character_name

        return creator_name

    _creator.short_description = "Creator"
    _creator.admin_order_field = "creator"


@admin.register(AaSrpRequest)
class AaSrpRequestAdmin(admin.ModelAdmin):
    """
    AaSrpRequestAdmin
    """

    list_display = (
        "request_code",
        "_creator",
        "character",
        "srp_link",
        "post_time",
        "ship",
        "loss_amount",
        "payout_amount",
        "killboard_link",
        "request_status",
    )
    ordering = ("post_time",)

    list_filter = ("creator", "character", "request_status")

    search_fields = (
        "request_code",
        "character__character_name",
        "ship__name",
        "srp_link__srp_code",
    )

    @classmethod
    def _creator(cls, obj):
        creator_name = obj.creator
        if obj.creator.profile.main_character:
            creator_name = obj.creator.profile.main_character.character_name

        return creator_name

    _creator.short_description = "Creator"
    _creator.admin_order_field = "creator"


@admin.register(AaSrpRequestComment)
class AaSrpRequestCommentAdmin(admin.ModelAdmin):
    """
    AaSrpRequestCommentAdmin
    """

    list_display = ("srp_request", "comment_type", "creator")
    ordering = ("srp_request",)
    list_filter = ("comment_type",)



"""
aasrp url config
"""

# Django
from django.urls import path

# AA SRP
from aasrp import views

app_name: str = "aasrp"

urlpatterns = [
    path("", views.dashboard, name="dashboard"),
    path("all/", views.dashboard, {"show_all_links": True}, name="all"),
    path("add/", views.srp_link_add, name="add_srp_link"),
    path("srp-link/<str:srp_code>/edit/", views.srp_link_edit, name="edit_srp_link"),
    path(
        "srp-link/<str:srp_code>/view-srp-requests/",
        views.srp_link_view_requests,
        name="view_srp_requests",
    ),
    path(
        "srp-link/<str:srp_code>/enable/", views.enable_srp_link, name="enable_srp_link"
    ),
    path(
        "srp-link/<str:srp_code>/disable/",
        views.disable_srp_link,
        name="disable_srp_link",
    ),
    path(
        "srp-link/<str:srp_code>/delete/", views.delete_srp_link, name="delete_srp_link"
    ),
    path("srp-link/<str:srp_code>/request-srp/", views.request_srp, name="request_srp"),
    path(
        "srp-link/<str:srp_code>/complete/",
        views.complete_srp_link,
        name="complete_srp_link",
    ),
    # ajax calls
    path(
        # get active srp links
        "call/active-srp-links-data/",
        views.ajax_dashboard_srp_links_data,
        name="ajax_dashboard_srp_links_data",
    ),
    path(
        # get all srp links
        "call/active-srp-links-data/all/",
        views.ajax_dashboard_srp_links_data,
        {"show_all_links": True},
        name="ajax_dashboard_srp_links_all_data",
    ),
    path(
        # get all srp request for the current user
        "call/user-srp-requests-data/",
        views.ajax_dashboard_user_srp_requests_data,
        name="ajax_dashboard_user_srp_requests_data",
    ),
    path(
        # get all srp requests for the current srp link
        "call/srp-link/<str:srp_code>/view-srp-requests-data/",
        views.ajax_srp_link_view_requests_data,
        name="ajax_srp_link_view_requests_data",
    ),
    path(
        # get addition information for the current srp request
        (
            "call/srp-link/<str:srp_code>/srp-request/"
            "<str:srp_request_code>/view-additional-information-data/"
        ),
        views.ajax_srp_request_additional_information,
        name="ajax_srp_request_additional_information",
    ),
    path(
        # change the srp payout amount
        (
            "call/srp-link/<str:srp_code>/srp-request/"
            "<str:srp_request_code>/change-srp-payout/"
        ),
        views.ajax_srp_request_change_payout,
        name="ajax_srp_request_change_payout",
    ),
    path(
        # change the srp payout amount
        "call/srp-link/<str:srp_code>/srp-request/<str:srp_request_code>/approve/",
        views.ajax_srp_request_approve,
        name="ajax_srp_request_approve",
    ),
    path(
        # change the srp payout amount
        "call/srp-link/<str:srp_code>/srp-request/<str:srp_request_code>/deny/",
        views.ajax_srp_request_deny,
        name="ajax_srp_request_deny",
    ),
    path(
        # change the srp payout amount
        "call/srp-link/<str:srp_code>/srp-request/<str:srp_request_code>/remove/",
        views.ajax_srp_request_remove,
        name="ajax_srp_request_remove",
    ),
]



"""
notifications helper
"""

# pylint: disable=import-outside-toplevel

# Django
from django.contrib.auth.models import User
from django.utils import timezone

# Alliance Auth
from allianceauth.notifications import notify
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA SRP
from aasrp import __title__
from aasrp.app_settings import aa_discordnotify_active, allianceauth_discordbot_active
from aasrp.constants import DISCORD_EMBED_COLOR_MAP

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


def _aadiscordbot_send_private_message(user_id: int, message: str) -> None:
    """
    Try to send a PM to a user on Discord via allianceauth-discordbot
    :param user_id:
    :param message:
    :return:
    """

    if allianceauth_discordbot_active():
        logger.debug(
            "allianceauth-discordbot is active, trying to send private message"
        )

        # Third Party
        import aadiscordbot.tasks

        aadiscordbot.tasks.send_direct_message_by_user_id.delay(user_id, message)


def _aadiscordbot_send_channel_message(channel_id: int, message: str) -> None:
    """
    Try to send a message to a channel on Discord via allianceauth-discordbot
    :param channel_id:
    :param message:
    :return:
    """

    if allianceauth_discordbot_active():
        logger.debug(
            "allianceauth-discordbot is active, trying to send channel message"
        )

        # Third Party
        import aadiscordbot.tasks

        aadiscordbot.tasks.send_channel_message_by_discord_id.delay(
            channel_id, message, False
        )


def send_user_notification(user: User, level: str, title: str, message: str) -> None:
    """
    Send notification to user
    This creates a notification in Auth and a PM in Discord when either
    Discordproxy, AA-Discordbot or AA Discord Notifications is installed
    :param user:
    :param level:
    :param title:
    :param message:
    """

    notify(user=user, title=title, level=level, message=message)

    # Handle Discord PMs when aa_discordnotify is not active
    # Check if either allianceauth_discordbot or discordproxy are available
    # to send the PM
    if not aa_discordnotify_active():
        try:
            logger.debug(
                "Trying to import libraries from discordproxy to send a direct message"
            )

            # Third Party
            from discordproxy.client import DiscordClient
            from discordproxy.discord_api_pb2 import Embed
            from discordproxy.exceptions import DiscordProxyException
        except ModuleNotFoundError:
            # discordproxy not available, try if allianceauth-discordbot is available
            logger.debug(
                "discordproxy not available to send a direct message, "
                "let's see if we can use allianceauth-discordbot"
            )
            _aadiscordbot_send_private_message(user.pk, f"**{title}**\n\n{message}")
        else:
            # discordproxy is available, use it,
            # if the user has a Discord account that is
            logger.debug(
                "discordproxy seems to be available, "
                "check if the user has a Discord account we can use"
            )

            if hasattr(user, "discord"):
                logger.debug("User has a Discord account")

                client = DiscordClient()

                footer = Embed.Footer(text=__title__)

                embed = Embed(
                    title=str(title),
                    description=message,
                    color=DISCORD_EMBED_COLOR_MAP.get(level, None),
                    timestamp=timezone.now().isoformat(),
                    footer=footer,
                )

                try:
                    logger.debug("Trying to send a direct message via discordproxy")

                    client.create_direct_message(
                        user_id=int(user.discord.uid), embed=embed
                    )
                except DiscordProxyException as ex:
                    # Something went wrong with discordproxy
                    # Fail silently and try if allianceauth-discordbot is available
                    # as a last ditch effort to get the message out to Discord
                    logger.debug(
                        "Something went wrong with discordproxy, "
                        "cannot send a direct message, "
                        f"trying allianceauth-discordbot to send the message. Error: {ex}"
                    )

                    _aadiscordbot_send_private_message(
                        user.pk, f"**{title}**\n\n{message}"
                    )
            else:
                logger.debug(
                    "User doesn't have a Discord account, can't send any messages"
                )


def send_message_to_discord_channel(
    channel_id: int, title: str, message: str, embed: bool = False
) -> None:
    """
    Sending a message to a discord channel
    This creates a message to the SRP Team channel on Discord when either
    Discordproxy or AA-Discordbot is installed
    :param channel_id:
    :param title:
    :param message:
    :param embed:
    """

    # Check if either allianceauth_discordbot or discordproxy are available
    # to send the channel message
    try:
        logger.debug(
            "Trying to import libraries from discordproxy to send channel messages"
        )

        # Third Party
        from discordproxy.client import DiscordClient
        from discordproxy.discord_api_pb2 import Embed
        from discordproxy.exceptions import DiscordProxyException
    except ModuleNotFoundError:
        # discordproxy not available, try if allianceauth-discordbot is available
        logger.debug(
            "discordproxy not available to send the channel, "
            "let's see if we can use allianceauth-discordbot"
        )

        _aadiscordbot_send_channel_message(channel_id, f"**{title}**\n\n{message}")
    else:
        # discordproxy is available, use it
        logger.debug("discordproxy seems to be available, check if we can use it")

        client = DiscordClient()

        footer = Embed.Footer(text=__title__)

        embed = Embed(
            title=title,
            description=message,
            color=DISCORD_EMBED_COLOR_MAP.get("info", None),
            timestamp=timezone.now().isoformat(),
            footer=footer,
        )

        try:
            logger.debug("Trying to send a channel message via discordproxy")

            client.create_channel_message(channel_id=channel_id, embed=embed)
        except DiscordProxyException as ex:
            # Something went wrong with discordproxy
            # Fail silently and try if allianceauth-discordbot is available
            # as a last ditch effort to get the message out to Discord
            logger.debug(
                "Something went wrong with discordproxy, "
                "cannot send a channel message, "
                f"trying allianceauth-discordbot to send the message. Error: {ex}"
            )

            _aadiscordbot_send_channel_message(channel_id, f"**{title}**\n\n{message}")



"""
SRP Manager
"""

# Third Party
import requests

# Django
from django.contrib.auth.models import User
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA SRP
from aasrp import __title__
from aasrp.constants import USERAGENT, ZKILLBOARD_API_URL
from aasrp.models import AaSrpRequest
from aasrp.providers import esi

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


class AaSrpManager:
    """
    AaSrpManager
    """

    @staticmethod
    def get_kill_id(killboard_link: str):
        """
        get killmail ID from zKillboard link
        :param killboard_link:
        :return:
        """

        num_set = "0123456789"
        kill_id = "".join(c for c in killboard_link if c in num_set)

        return kill_id

    @staticmethod
    def get_kill_data(kill_id: str):
        """
        get kill data from zKillboard
        :param kill_id:
        :return:
        """

        url = f"{ZKILLBOARD_API_URL}killID/{kill_id}/"

        headers = {
            "User-Agent": USERAGENT,
            "Content-Type": "application/json",
        }

        try:
            request_result = requests.get(url, headers=headers)
            result = request_result.json()[0]
        except IndexError:
            raise ValueError("Invalid Kill ID")

        if result:
            killmail_id = result["killmail_id"]
            killmail_hash = result["zkb"]["hash"]

            esi_killmail = esi.client.Killmails.get_killmails_killmail_id_killmail_hash(
                killmail_id=killmail_id, killmail_hash=killmail_hash
            ).result()
        else:
            raise ValueError("Invalid Kill ID")

        if esi_killmail:
            ship_type = esi_killmail["victim"]["ship_type_id"]
            logger.debug(f"Ship type for kill ID {kill_id} is {ship_type}")
            ship_value = result["zkb"]["totalValue"]

            logger.debug(f"Total loss value for kill id {kill_id} is {ship_value}")

            victim_id = esi_killmail["victim"]["character_id"]

            return ship_type, ship_value, victim_id

        raise ValueError(_("Invalid Kill ID or Hash."))

    @staticmethod
    def pending_requests_count_for_user(user: User):
        """
        returns the number of open SRP requests for given user
        or None if user has no permission
        """

        if user.has_perm("aasrp.manage_srp") or user.has_perm(
            "aasrp.manage_srp_requests"
        ):
            return AaSrpRequest.objects.filter(
                request_status=AaSrpRequest.Status.PENDING
            ).count()

        return None

    @staticmethod
    def get_insurance_for_ship_type(ship_type_id: int):
        """
        getting insurance for a given ship type ID from ESI
        :param ship_type_id:
        :type ship_type_id:
        """

        insurance_prices = esi.client.Insurance.get_insurance_prices().result()

        for insurance in insurance_prices:
            if insurance["type_id"] == ship_type_id:
                return insurance

        return None



"""
some helper functions
so we don't mess up other files too much
"""

# Django
from django.contrib.auth.models import User
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.eveonline.models import EveCharacter

# AA SRP
from aasrp.helper.eve_images import get_character_portrait_from_evecharacter
from aasrp.models import get_sentinel_user


def get_formatted_character_name(
    character: EveCharacter,
    with_portrait: bool = False,
    with_copy_icon: bool = False,
    portrait_size: int = 32,
    inline: bool = True,
) -> str:
    """
    get character name with alliance and corp ticker
    :param character:
    :param with_portrait:
    :param with_copy_icon:
    :param portrait_size:
    :param inline:
    """

    try:
        character_name = character.character_name
    except AttributeError:
        character_name = _("Unknown Character")

        return (
            f"<span class='aasrp-character-portrait-character-name'>"
            f"{character_name}"
            f"</span>"
        )
    else:
        character__corporation_ticker = (
            f"[{character.corporation_ticker}] " if character.corporation_ticker else ""
        )
        character__alliance_ticker = (
            f"{character.alliance_ticker} " if character.alliance_ticker else ""
        )
        character_name_formatted = (
            f"<small class='text-muted'>"
            f"{character__alliance_ticker}{character__corporation_ticker}</small>"
            f"<br>{character_name}"
        )

        if with_copy_icon is True:
            title = _("Copy character name to clipboard")
            character_name_formatted += (
                f"<i "
                f'class="aa-srp-fa-icon aa-srp-fa-icon-right copy-text-fa-icon far fa-copy" '
                f'data-clipboard-text="{character_name}" title="{title}"></i>'
            )

        return_value = character_name_formatted

        if with_portrait is True:
            line_break = ""
            if inline is False:
                line_break = "<br>"

            character_portrait_html = get_character_portrait_from_evecharacter(
                character=character, size=portrait_size, as_html=True
            )

            return_value = (
                f"{character_portrait_html}{line_break}"
                f"<span class='aasrp-character-portrait-character-name'>"
                f"{character_name_formatted}"
                f"</span>"
            )

    return return_value


def get_main_for_character(character: EveCharacter) -> EveCharacter:
    """
    get the main character for a given eve character
    :param character:
    """

    return_value = None

    if character.userprofile:
        return_value = character.userprofile.main_character

    return return_value


def get_user_for_character(character: EveCharacter) -> User:
    """
    get the user for a character
    :param character:
    :return:
    """

    if character.userprofile is None:
        return_value = get_sentinel_user()
    else:
        return_value = character.userprofile.user

    return return_value


def get_main_character_from_user(user: User) -> str:
    """
    Get the main character from a user
    :param user:
    :type user:
    :return:
    :rtype:
    """

    user_main_character = user.username

    try:
        user_profile = user.profile
        user_main_character = user_profile.main_character.character_name
    except AttributeError:
        pass

    return user_main_character



"""
hook into AA
"""

# Django
from django.utils.translation import ugettext_lazy as _

# Alliance Auth
from allianceauth import hooks
from allianceauth.services.hooks import MenuItemHook, UrlHook

# AA SRP
from aasrp import __title__, urls
from aasrp.managers import AaSrpManager


class AaSrpMenuItem(MenuItemHook):  # pylint: disable=too-few-public-methods
    """
    This class ensures only authorized users will see the menu entry
    """

    def __init__(self):
        # setup menu entry for sidebar
        MenuItemHook.__init__(
            self,
            _(__title__),
            "far fa-money-bill-alt fa-fw",
            "aasrp:dashboard",
            navactive=["aasrp:"],
        )

    def render(self, request):
        """
        check if the user has the permission to view this app
        :param request:
        :return:
        """

        if request.user.has_perm("aasrp.basic_access"):
            app_count = AaSrpManager.pending_requests_count_for_user(request.user)
            self.count = app_count if app_count and app_count > 0 else None

            return MenuItemHook.render(self, request)

        return ""


@hooks.register("menu_item_hook")
def register_menu():
    """
    register our menu item
    :return:
    """

    return AaSrpMenuItem()


@hooks.register("url_hook")
def register_urls():
    """
    register our basu url
    :return:
    """

    return UrlHook(urls, "aasrp", r"^ship-replacement/")
